#region GNU_GPL
// Copyright (c) 2003 The Ohio State University
//
// Anish Arora <anish@cis.ohio-state.edu>
// Jason O. Hallstrom <hallstro@cis.ohio-state.edu>
// William M. Leal <leal@cis.ohio-state.edu>
//
// This file is part of DVI.
//
// DVI is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// DVI is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with DVI; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#endregion


using System;
using System.Drawing;
using System.Collections;
using System.Windows.Forms;
using System.ComponentModel;
using OSU.CallTracking;
using OSU.DVI.GUI.Rendering;


namespace OSU.DVI.GUI.CallTracking
{
	
	public class CallTraceControl : UserControl
	{
		#region Fields
		//-----------------------------------------------------------------
		// Fields
		//-----------------------------------------------------------------
		// This field is required by the Windows Control Designer
		private Container components = null;

		// The length of the logical trace window (in milliseconds)
		private long lonTraceLength;

		// The underlying array of call log entries rendered by the control
		private ArrayList alLogEntries;

		// The hashtable mapping (rendering) center point to log entry.  This map 
		// is used to locate the log entry corresponding to the point where the user 
		// clicks
		private Hashtable htPointToLogEntry;

		// The dialog used to display method call information when the user clicks on
		// a method call bubble
		private MethodCallDialog dlgMethodCallDialog;
		//-----------------------------------------------------------------
		#endregion


		#region Constructors
		//-----------------------------------------------------------------
		// Constructors
		//-----------------------------------------------------------------
		public CallTraceControl()
		{
			// This call is required by the Windows Control Designer.
			InitializeComponent();

			// Set a default trace window of 30 seconds
			this.lonTraceLength = 30000;
			this.alLogEntries = new ArrayList();
			this.htPointToLogEntry = new Hashtable();
			this.dlgMethodCallDialog = new MethodCallDialog();
		}
		//-----------------------------------------------------------------
		#endregion


		#region Properties
		//-----------------------------------------------------------------
		// Properties
		//-----------------------------------------------------------------
		public long TraceLength
		{
			get
			{	return(lonTraceLength);    }

			set
			{	
				lonTraceLength = value;    
				this.Invalidate();
				this.Update();
			}
		}
		//-----------------------------------------------------------------
		#endregion


		#region Event Handlers
		//-----------------------------------------------------------------
		// Event Handlers
		//-----------------------------------------------------------------
		private void CallTraceControl_Resize(object objSource, EventArgs eaArgs)
		{	
			this.Invalidate();
			this.Update();
		}
		
		private void CallTraceControl_MouseDown(object objSource, MouseEventArgs meaArgs)
		{
			lock(htPointToLogEntry)
			{
				// Check to see which circle(s) the user clicked inside of
				int intCircleRadius = this.Height / 5;
				ICollection colCenterPoints =  htPointToLogEntry.Keys;
				foreach(Point pntCenter in colCenterPoints)
				{
					// Calculate the distance between the center of the current circle and the 
					// point where the user clicked
					int intDistance = (int) Math.Sqrt(Math.Pow(pntCenter.X - meaArgs.X, 2.0) +
											Math.Pow(pntCenter.Y - meaArgs.Y, 2.0));
					
					// Test whether the user clicked inside the current circle
					if(intDistance <= intCircleRadius)
					{	
						// Lookup and display the selected call log entry
						CallLogEntry cleLogEntry = (CallLogEntry) htPointToLogEntry[pntCenter];
						dlgMethodCallDialog.Show(cleLogEntry);
					}
				}
			}
		 }
		//-----------------------------------------------------------------
		#endregion

		
		#region Methods
		//-----------------------------------------------------------------
		// Methods
		//-----------------------------------------------------------------
		public void UpdateTrace(ICollection colLogEntries)
		{	
			lock(alLogEntries)
			{	alLogEntries.AddRange(colLogEntries);    }
		}

		public void ClearTrace()
		{
			lock(alLogEntries)
			{	alLogEntries.Clear();    }
		}

		protected override void OnPaint(PaintEventArgs peaArgs)
		{
			base.OnPaint(peaArgs);
		
			Graphics grpCanvas = peaArgs.Graphics;

			// Set the font / pen / brush
			Font fntFont = new Font("Sans Serif", 4f);
			SolidBrush brsBrush = new SolidBrush(this.BackColor);
			Pen penPen = new Pen(this.ForeColor, 1.0f);
			
			// Draw the ruler 
			Rectangle recBorder = new Rectangle(new Point(0, 0), new Size(this.Width - 1, this.Height - 1));
			ShapeRenderingUtilities.DrawRuler(grpCanvas, penPen, brsBrush, fntFont, recBorder, lonTraceLength, 1000); 

			int intCircleDiameter = this.Height / 3;			
			int intCircleY = (this.Height / 2) - (intCircleDiameter / 2);
	
			//---
			// Render the method calls
			long lonWindowTop =	(long) ((TimeSpan) (DateTime.Now - new DateTime(2000, 1, 1))).TotalMilliseconds;
			long lonWindowBottom = lonWindowTop - lonTraceLength;
			lock(alLogEntries)
			{
				lock(htPointToLogEntry)
				{
					htPointToLogEntry.Clear();
					ArrayList alProcessedMethodCalls = new ArrayList(alLogEntries.Count);
					while(alLogEntries.Count > 0)
					{
						CallLogEntry cleLogEntry = (CallLogEntry) alLogEntries[0];
						alLogEntries.RemoveAt(0);

						// Filter out method calls that lie outside the rendering window
						long lonCallTime = (long) ((TimeSpan) (cleLogEntry.Timestamp - new DateTime(2000, 1, 1))).TotalMilliseconds;
						if((lonCallTime >= lonWindowBottom) && (lonCallTime <= lonWindowTop))
						{	
							// Set the fill color of the circle based on the method name
							brsBrush.Color = Color.FromArgb(cleLogEntry.CallInformation.MethodName.GetHashCode());
						
							// Draw the circle at the appropriate point on the ruler
							double dblPercent = (double) (lonCallTime - lonWindowBottom) / (lonWindowTop - lonWindowBottom);
							int intCircleX = (int) ((double) (1.0 - dblPercent) * (this.Width - 1.0)) - (intCircleDiameter / 2);
							grpCanvas.FillEllipse(brsBrush, intCircleX, intCircleY, intCircleDiameter, intCircleDiameter);
							grpCanvas.DrawEllipse(penPen, intCircleX, intCircleY, intCircleDiameter, intCircleDiameter);
						
							// Add an entry mapping from the circle center point to the call log entry.
							// (This data supports the mouse-down event handler)
							Point pntCircleCenter = new Point(intCircleX, intCircleY);
							if(!htPointToLogEntry.ContainsKey(pntCircleCenter))
							{	htPointToLogEntry.Add(pntCircleCenter, cleLogEntry);    }
							
							alProcessedMethodCalls.Add(cleLogEntry);
						}
					}
					
					alLogEntries.AddRange(alProcessedMethodCalls);
				}
			}
			//---
		}

		protected override void Dispose(bool bolDisposing)
		{
			if((bolDisposing) && (components != null))
			{	components.Dispose();    }
			
			base.Dispose(bolDisposing);
		}
		//-----------------------------------------------------------------
		#endregion


		#region Component Designer Generated Code
		//-----------------------------------------------------------------
		// Component Designer Generated Code
		//-----------------------------------------------------------------
		// Required method for Windows Component Designer support - do not 
		// modify the contents of this method with the code editor.
		//-----------------------------------------------------------------
		private void InitializeComponent()
		{
			// 
			// CallTraceControl
			// 
			this.Name = "CallTraceControl";
			this.Size = new System.Drawing.Size(264, 32);
			this.Resize += new System.EventHandler(this.CallTraceControl_Resize);
			this.MouseDown += new System.Windows.Forms.MouseEventHandler(this.CallTraceControl_MouseDown);

		}
		//-----------------------------------------------------------------
		#endregion
	}

}
