#region GNU_GPL
// Copyright (c) 2003 The Ohio State University
//
// Anish Arora <anish@cis.ohio-state.edu>
// Jason O. Hallstrom <hallstro@cis.ohio-state.edu>
// William M. Leal <leal@cis.ohio-state.edu>
//
// This file is part of DVI.
//
// DVI is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// DVI is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with DVI; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#endregion


using System;
using System.Drawing;
using System.Threading;
using System.Collections;
using System.Windows.Forms;
using System.ComponentModel;
using OSU.DRSS.Util;
using OSU.CallTracking;


namespace OSU.DVI.GUI.CallTracking
{
	
	public class CallTrackingPageControl : UserControl
	{
		#region Fields
		//-----------------------------------------------------------------
		// Fields
		//-----------------------------------------------------------------
		// The maximum number of traces supported by the control
		public const int intMaximumTraceCount = 15;

		// The number of milliseconds between control updates
		public const int intRefreshDelay = 500;

		// The logical width of the control
		public const int intControlWidth = 500;

		// The top margin for laying out labels and call traces
		public const int intTopMargin = 10;
		
		// The height of the label controls
		public const int intLabelHeight = 16;
		
		// The gap between each label and call trace control
		public const int intInternalGap = 5;
		
		// The height of the call-trace controls
		public const int intTraceHeight = 40;

		// The gap between label / call-trace pairs
		public const int intExternalGap = 15;
		
		// This field is required by the Windows Control Designer
		private IContainer components = null;
		
		// The call-tracking popup menu, and the associated menu items
		private ContextMenu cmCallTrackingPopup;
		private MenuItem miClear;
		
		// The array of label controls
		private Label[] lblTargetIDs;

		// The array of call-trace controls
		private CallTraceControl[] ctcCallTraces;

		// The index of the next available label / call-trace
		private int intNextAvailableControl;

		// The hashtable mapping each object ID (GUID) to its corresponding 
		// call-trace control
		private Hashtable htTargetIDToCallTrace;
		
		// The call-log that receives call log entries from remote processes
		private CallLog clCallLog;
		//-----------------------------------------------------------------
		#endregion


		#region Constructors
		//-----------------------------------------------------------------
		// Constructors
		//-----------------------------------------------------------------
		public CallTrackingPageControl()
		{
			// This call is required by the Windows Form Designer.
			InitializeComponent();
			
			this.lblTargetIDs = new Label[intMaximumTraceCount];
			this.ctcCallTraces = new CallTraceControl[intMaximumTraceCount];
			this.intNextAvailableControl = 0;
			this.htTargetIDToCallTrace = new Hashtable();

			//---
			// Setup the (initially invisible) label and call trace controls
			int intNextY = intTopMargin;
			for(int intIndex = 0; intIndex < lblTargetIDs.Length; intIndex++)
			{
				// Create (and add) the label
				Label lblTargetID = new Label();
				lblTargetID.Font = new Font("Sans Serif", 8.25f, FontStyle.Underline);
				lblTargetID.Height = intLabelHeight;
				lblTargetID.Width = intControlWidth;
				lblTargetID.Location = new Point(0, intNextY);
				intNextY += intLabelHeight + intInternalGap;
				this.lblTargetIDs[intIndex] = lblTargetID;
				this.Controls.Add(lblTargetID);

				// Create (and add) the call-trace control
				CallTraceControl ctcCallTrace = new CallTraceControl();
				ctcCallTrace.Height = intTraceHeight;
				ctcCallTrace.Width = intControlWidth;
				ctcCallTrace.Location = new Point(0, intNextY);
				this.ctcCallTraces[intIndex] = ctcCallTrace;
				this.Controls.Add(ctcCallTrace);
				intNextY += intTraceHeight + intExternalGap;
			}
			//---
		}
		//-----------------------------------------------------------------
		#endregion


		#region Event Handlers
		//-----------------------------------------------------------------
		// Event Handlers
		//-----------------------------------------------------------------
		private void miClear_Click(object objSource, EventArgs eaArgs)
		{
			lock(htTargetIDToCallTrace)
			{	
				// Reset the control state
				for(int intIndex = 0; intIndex < intNextAvailableControl; intIndex++)
				{	
					lblTargetIDs[intIndex].Text = "";    
					ctcCallTraces[intIndex].ClearTrace();
				}
				intNextAvailableControl = 0;
				htTargetIDToCallTrace.Clear();
			}
		}
		//-----------------------------------------------------------------
		#endregion


		#region Methods
		//-----------------------------------------------------------------
		// Methods
		//-----------------------------------------------------------------
		public void InitializeControl(CallLog clCallLog)
		{
			this.clCallLog = clCallLog;
			
			// Startup the control refresh thread
			ThreadStart tsRefreshThread = new ThreadStart(RefreshThreadBody);
			Thread thdRefreshThread = new Thread(tsRefreshThread);
			thdRefreshThread.IsBackground = true;
			thdRefreshThread.Start();
		}

		private void RefreshThreadBody()
		{
			while(true)
			{
				Thread.Sleep(intRefreshDelay);

				// Process (new) calls that have been logged since the last refresh
				Guid[] guiFreshTargetIDs = clCallLog.TargetIDs;
				for(int intIndex = 0; intIndex < guiFreshTargetIDs.Length; intIndex++)
				{
					lock(htTargetIDToCallTrace)
					{
						// If a trace control has not been assigned to the current target ID, 
						// assign a label and call-trace
						bool bolSkipTarget = false;
						Guid guiFreshTargetID = guiFreshTargetIDs[intIndex];
						if(!htTargetIDToCallTrace.ContainsKey(guiFreshTargetID))
						{
							// Skip the target if there are not enough label / call-trace controls
							// available
							if(intNextAvailableControl < intMaximumTraceCount)
							{
								// Assign a label / call-trace
								Label lblTargetID = lblTargetIDs[intNextAvailableControl];
								CallTraceControl ctcCallTrace = ctcCallTraces[intNextAvailableControl];
								intNextAvailableControl++;

								// Setup the label
								lblTargetID.Text = guiFreshTargetID.ToString();
							
								// Add the object ID / call-trace entry pair to the hashtable map
								htTargetIDToCallTrace.Add(guiFreshTargetID, ctcCallTrace);
							}
							else
							{	bolSkipTarget = true;    }
						}

						// If the target wasn't skipped, deliver the new call log entries to the 
						// appropriate trace control
						if(!bolSkipTarget)
						{
							ICollection colLogEntries = clCallLog.RemoveLogEntries(guiFreshTargetID);	
							((CallTraceControl) htTargetIDToCallTrace[guiFreshTargetID]).UpdateTrace(colLogEntries);
						}
					}
				}
			
				this.Refresh();
			}
		}

		protected override void Dispose(bool bolDisposing)
		{
			if((bolDisposing) && (components != null))
			{	components.Dispose();    }
			
			base.Dispose(bolDisposing);
		}
		//-----------------------------------------------------------------
		#endregion


		#region Component Designer Generated Code
		//-----------------------------------------------------------------
		// Component Designer Generated Code
		//-----------------------------------------------------------------
		// Required method for Windows Control Designer support - do not modify 
		// the contents of this method with the code editor.
		//-----------------------------------------------------------------
		private void InitializeComponent()
		{
			this.cmCallTrackingPopup = new System.Windows.Forms.ContextMenu();
			this.miClear = new System.Windows.Forms.MenuItem();
			// 
			// cmCallTrackingPopup
			// 
			this.cmCallTrackingPopup.MenuItems.AddRange(new System.Windows.Forms.MenuItem[] {
																								this.miClear});
			// 
			// miClear
			// 
			this.miClear.Index = 0;
			this.miClear.Text = "&Clear";
			this.miClear.Click += new System.EventHandler(this.miClear_Click);
			// 
			// CallTrackingPageControl
			// 
			this.AutoScroll = true;
			this.AutoScrollMargin = new System.Drawing.Size(5, 5);
			this.ContextMenu = this.cmCallTrackingPopup;
			this.Location = new System.Drawing.Point(4, 3);
			this.Name = "CallTrackingPageControl";
			this.Size = new System.Drawing.Size(328, 219);
		}
		//-----------------------------------------------------------------
		#endregion
	}

}
