#region GNU_GPL
// Copyright (c) 2003 The Ohio State University
//
// Anish Arora <anish@cis.ohio-state.edu>
// Jason O. Hallstrom <hallstro@cis.ohio-state.edu>
// William M. Leal <leal@cis.ohio-state.edu>
//
// This file is part of DVI.
//
// DVI is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// DVI is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with DVI; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#endregion


using System;
using System.Drawing;
using System.Threading;
using System.Collections;
using System.ComponentModel;
using System.Windows.Forms;
using OSU.DRSS.Util;


namespace OSU.DVI.GUI.ManagedObjects
{
	
	public class RefreshableManagedObjectListViewControl : UserControl
	{
		#region Fields
		//-----------------------------------------------------------------
		// Fields
		//-----------------------------------------------------------------
		// This field is required by the Windows Control Designer
		private Container components = null;
		
		// The basic 'Managed Objects' list-view
		private ManagedObjectListViewControl mlvManagedObjects;
		
		// The 'Refresh' button
		private Button btnRefresh;

		// The string to prepend to all event messages generated by this control
		private string strEventMessagePrefix;

		// The managed object producer (delegate) to use during list-view 
		// refresh operations
		private ManagedObjectProducerDelegate DLG_ProduceManagedObjects;
		//-----------------------------------------------------------------
		#endregion


		#region Constructors
		//-----------------------------------------------------------------
		// Constructors
		//-----------------------------------------------------------------
		public RefreshableManagedObjectListViewControl()
		{
			// This call is required by the Windows Control Designer.
			InitializeComponent();

			this.strEventMessagePrefix = null;
			this.DLG_ProduceManagedObjects = null;
		}
		//-----------------------------------------------------------------
		#endregion


		#region Properties
		//-----------------------------------------------------------------
		// Properties
		//-----------------------------------------------------------------
		public ListView.SelectedListViewItemCollection SelectedItems
		{
			get
			{	return(mlvManagedObjects.SelectedItems);    }
		}
		//-----------------------------------------------------------------
		#endregion

		
		#region Event Handlers
		//-----------------------------------------------------------------
		// Event Handlers
		//-----------------------------------------------------------------
		private void btnRefresh_Click(object objSource, EventArgs eaArgs)
		{	RefreshManagedObjects();    }
		//-----------------------------------------------------------------
		#endregion

		
		#region Methods
		//-----------------------------------------------------------------
		// Methods
		//-----------------------------------------------------------------
		public virtual void InitializeControl(ContextMenu cmManagedObjectsPopup, string strEventMessagePrefix, ManagedObjectProducerDelegate dlgManagedObjectProducer)
		{
			this.mlvManagedObjects.InitializeControl(cmManagedObjectsPopup);
			this.strEventMessagePrefix = strEventMessagePrefix;
			this.DLG_ProduceManagedObjects = dlgManagedObjectProducer;
			
			// Refresh the list of managed objects
			RefreshManagedObjects();
		}

		private void RaiseSystemEvent(string strMessage)
		{
			if(SystemEvent != null)
			{	
				string strFormattedMessage = (strEventMessagePrefix != null) ? strEventMessagePrefix + ": " + strMessage : strMessage;				
				SystemEvent(this, new SystemEventArgs(strFormattedMessage));    
			}
		}
		
		private void RefreshManagedObjects()
		{
			// Disable the 'Refresh' button
			btnRefresh.Enabled = false;
			
			// Start the refresh thread
			ThreadStart tsRefreshManagedObjects = new ThreadStart(RefreshManagedObjectsThreadBody);
			Thread thdRefreshManagedObjectsThread = new Thread(tsRefreshManagedObjects);
			thdRefreshManagedObjectsThread.IsBackground = true;
			thdRefreshManagedObjectsThread.Start();
		}

		private void RefreshManagedObjectsThreadBody()
		{
			try
			{
				// Refresh the list-view with a new list of managed objects
				// (produced by the managed object producer)
				IEnumerator enmManagedObjects = DLG_ProduceManagedObjects();
				mlvManagedObjects.RefreshManagedObjects(enmManagedObjects);
				RaiseSystemEvent("View last refreshed " + DateTime.Now + ".");
			}
			catch(Exception exc)
			{	RaiseSystemEvent(exc.ToString());    }

			// Enable the 'Refresh' button
			btnRefresh.Enabled = true;
		}

		protected override void Dispose(bool bolDisposing)
		{
			if((bolDisposing) && (components != null))
			{	components.Dispose();    }
			
			base.Dispose(bolDisposing);
		}
		//-----------------------------------------------------------------
		#endregion


		#region Events
		//-----------------------------------------------------------------
		// Events
		//-----------------------------------------------------------------
		// This event is raised internally whenever something of interest 
		// occurs within the refresh thread
		public event SystemEventDelegate SystemEvent;
		//-----------------------------------------------------------------
		#endregion


		#region Component Designer Generated Code
		//-----------------------------------------------------------------
		// Component Designer Generated Code
		//-----------------------------------------------------------------
		// Required method for Windows Control Designer support - do not 
		// modify the contents of this method with the code editor.
		//-----------------------------------------------------------------
		private void InitializeComponent()
		{
			this.mlvManagedObjects = new OSU.DVI.GUI.ManagedObjects.ManagedObjectListViewControl();
			this.btnRefresh = new System.Windows.Forms.Button();
			this.SuspendLayout();
			// 
			// mlvManagedObjects
			// 
			this.mlvManagedObjects.Anchor = (((System.Windows.Forms.AnchorStyles.Top | System.Windows.Forms.AnchorStyles.Bottom) 
				| System.Windows.Forms.AnchorStyles.Left) 
				| System.Windows.Forms.AnchorStyles.Right);
			this.mlvManagedObjects.Location = new System.Drawing.Point(8, 8);
			this.mlvManagedObjects.Name = "mlvManagedObjects";
			this.mlvManagedObjects.Size = new System.Drawing.Size(240, 168);
			this.mlvManagedObjects.TabIndex = 0;
			// 
			// btnRefresh
			// 
			this.btnRefresh.Anchor = ((System.Windows.Forms.AnchorStyles.Bottom | System.Windows.Forms.AnchorStyles.Left) 
				| System.Windows.Forms.AnchorStyles.Right);
			this.btnRefresh.Enabled = false;
			this.btnRefresh.Location = new System.Drawing.Point(16, 176);
			this.btnRefresh.Name = "btnRefresh";
			this.btnRefresh.Size = new System.Drawing.Size(225, 23);
			this.btnRefresh.TabIndex = 3;
			this.btnRefresh.Text = "Refresh";
			this.btnRefresh.Click += new System.EventHandler(this.btnRefresh_Click);
			// 
			// RefreshableManagedObjectListViewControl
			// 
			this.Controls.AddRange(new System.Windows.Forms.Control[] {
																		  this.btnRefresh,
																		  this.mlvManagedObjects});
			this.Name = "RefreshableManagedObjectListViewControl";
			this.Size = new System.Drawing.Size(256, 208);
			this.ResumeLayout(false);

		}
		//-----------------------------------------------------------------
		#endregion
	}

}
